# FITS Tools

## FITS metainformation

```@docs
fits_info(hdu::FITS_HDU; nr=true, msg=true)
fits_record_dump(filnam::String, hduindex=0; hdr=true, dat=true, nr=true, msg=true)
```

## FITS creation, extension and collection

```@docs
fits_create(filnam::String, data=nothing; protect=true, msg=true)
fits_extend!(f::FITS, data_extend; hdutype="IMAGE")
fits_collect(filnamFirst::String, filnamLast::String; protect=true)
```

## FITS reading, copying and saving
```@docs
fits_read(filnam::String)
fits_save_as(f::FITS, filnam::String; protect=true)
fits_copy(fileStart::String, fileStop::String=" "; protect=true)
```

## FITS keyword tools

```@docs
fits_add_key!(f::FITS, hduindex::Int, key::String, val::Any, com::String)
fits_delete_key!(f::FITS, hduindex::Int, key::String)
fits_edit_key!(f::FITS, hduindex::Int, key::String, val::Real, com::String)
fits_rename_key!(f::FITS, hduindex::Int, keyold::String, keynew::String)
```

## FITS keyword tools

```@docs
fits_zero_offset(T::Type)::Real
```

## FITS keyword description tool

In the *CamiFITS* package the *mandatory FITS keywords* are *autogenerated*. 
To support easy user access to the definition of all *reserved FITS keywords* 
(mandatory or non-mandatory) *CamiFITS* includes the [`fits_keyword`](@ref) 
tool. 

The description of the *FITS keywords* is provided by [`fits_keyword`](@ref):
```
julia> fits_keyword("bitpix")
KEYWORD:    BITPIX
REFERENCE:  FITS Standard - version 4.0 - Appendix C
CLASS:      general
STATUS:     mandatory
HDU:        primary, groups, extension, array, image, ASCII-table, bintable,
VALUE:      integer
RANGE:      -64,-32,8,16,32,64
COMMENT:    bits per data value
DEFINITION: The value field shall contain an integer.  The absolute value is
used in computing the sizes of data structures. It shall specify the number of
bits that represent a data value (using a minus sign for floating point data).
```
Without argument [`fits_keyword`](@ref) provides the list of all 
FITS defined keywords (for the HDU types inplemented).
```
julia> fits_keyword()
FITS defined keywords:
(blanks) AUTHOR   BITPIX   BLANK    BLOCKED  BSCALE   BUNIT    BZERO            
CDELTn   COMMENT  CROTAn   CRPIXn   CRVALn   CTYPEn   DATAMAX  DATAMIN          
DATE     DATE-OBS END      EPOCH    EQUINOX  EXTEND   EXTLEVEL EXTNAME          
EXTVER   GCOUNT   GROUPS   HISTORY  INSTRUME NAXIS    NAXISn   OBJECT           
OBSERVER ORIGIN   PCOUNT   PSCALn   PTYPEn   PZEROn   REFERENC SIMPLE           
TBCOLn   TDIMn    TDISPn   TELESCOP TFIELDS  TFORMn   THEAP    TNULLn           
TSCALn   TTYPEn   TUNITn   TZEROn   XTENSION 

HDU options: 'primary', 'extension', 'array', 'image', 'ASCII-table', 'bintable'

reference: FITS Standard - version 4.0 - Appendix C
```
Specifying the *FITS HDU type* in [`fits_keyword`](@ref) the user obtains the 
restricted set of HDU-specific keywords.
```
julia> fits_keyword(hdutype="'PRIMARY '")
FITS defined keywords:
HDU type: 'primary'
- general
  - mandatory: BITPIX   END      NAXIS    NAXISn   SIMPLE
  - reserved : BLANK    BSCALE   BUNIT    BZERO    CDELTn   CROTAn   CRPIXn   
               CRVALn   CTYPEn   DATAMAX  DATAMIN  EXTEND
- bibliographic
  - mandatory:
  - reserved : AUTHOR   REFERENC
- commentary
  - mandatory:
  - reserved : (blanks) COMMENT  HISTORY
- observation
  - mandatory:
  - reserved : DATE-OBS EPOCH    EQUINOX  INSTRUME OBJECT   OBSERVER TELESCOP

HDU options: 'primary', 'extension', 'array', 'image', 'ASCII-table', 'bintable'

reference: FITS Standard - version 4.0 - Appendix C
```
By using the keyword "ALL" the user can dump the full list of keyword 
descriptions:
```
julia> fits_keyword("all")
FITS defined keywords:

KEYWORD:    (blank)
REFERENCE:  FITS Standard - version 4.0 - Appendix C
CLASS:      commentary
STATUS:     reserved
HDU:        primary, groups, extension, array, image, ASCII-table, bintable,
VALUE:      none
COMMENT:    descriptive comment
DEFINITION: Columns 1-8 contain ASCII blanks. This keyword has no associated 
value. Columns 9-80 may contain any ASCII text.  Any number of card images 
with blank keyword fields may appear in a header.
⋮
KEYWORD:    XTENSION
REFERENCE:  FITS Standard - version 4.0 - Appendix C
CLASS:      general
STATUS:     mandatory
HDU:        extension, array, image, ASCII-table, bintable,
VALUE:      string
COMMENT:    marks beginning of new HDU
DEFINITION: The value field shall contain a character string giving the name of 
the extension type. This keyword is mandatory for an extension header and must 
not appear in the primary header. For an extension that is not a standard 
extension, the type name must not be the same as that of a standard extension.

```

```@docs
fits_keyword(keyword::String; msg=true)
```

## FITS terminology

```@docs
fits_terminology(term::String; test=false)
```